#!/usr/bin/perl

# Use this script to rename your TLIB library files if you change TLIB's
# "libext" configuration parameter.
#
# This Perl script will rename all the TLIB library files in a specified
# directory or tree of directories, which were named according to the
# $old_libext convention, so that they will, instead, be named according
# to the $new_libext convention.
#
# (C) Copyright 2004, Burton Systems Software
# http://www.burtonsys.com/


# These are the usual conventions; you can change them as necessary:
$old_libext = '?$?';
$new_libext = '??????$';


# This script should work with 32-bit Windows versions of both Perl 4 & Perl 4.
# It has been tested with:
#
#    perl4.exe    379,392   5-25-1996  7:10a    ("jperl 4.036")
#
#    Perl.exe      20,480   8-20-2001 12:27p    ("ActiveState perl v5.6.1")
#    Perl56.dll   663,552   8-20-2001 12:27p
#
# This script will NOT work with perl4s.exe for MS-DOS unless you first do:
# set TLIBCFG=!C:\RENLIBS.CFG



$| = 1;  # don't buffer output


$testmode = 0;  # set by the '-t' option
$recursive = 0;  # set by the '-r' option
$quiet = 0;  # set by the '-q' option
$verbose = 0;  # set by the '-v' option


# for command-line syntax errors, display this help message and abort
sub givehelp {
   print "RenLibs.pl -- Rename TLIB library files for new LIBEXT convention\n" .
         "Copyright 2004, Burton Systems Software.  http://www.burtonsys.com/\n" .
         "\n" .
         "To use this script, first examine lines 16-17 and make sure that \$old_libext\n" .
         "and \$new_libext correctly reflect your old and new LIBEXT configuration\n" .
         "parmaeters.  Then run this script like this:\n" .
         "\n" .
         "  perl4 renlibs.pl {options} {directories}\n" .
         "\n" .
         "Or, if you have Perl 5 installed on your computer:\n" .
         "\n" .
         "  perl renlibs.pl {options} {directories}\n" .
         "\n" .
         "where {directories} are one or more paths to folders containing TLIB library\n" .
         "files, and {options} can be any combination of the following:\n" .
         "\n" .
         "  -t   test (just show what would be done, don't actually do it)\n" .
         "  -r   recursively traverse all the subdirectories beneath each directory\n" .
         "  -q   quieter  (use this with -t to generate a .bat file to stdout)\n" .
         "  -v   verbose (for debugging)\n" .
         "\n" .
         "Examples:\n" .
         "  \"perl4 renlibs.pl .\"   to convert tlib libraries in the current folder.\n" .
         "  \"perl4 renlibs.pl -r .\"   to convert tlib libraries in a tree of folders.\n" .
         "  \"perl4 renlibs.pl -rtq . >doit.bat\"   to generate a .bat file to do renames.\n";
   exit 0;
}#givehelp


# process one command-line option character
sub see_an_option {
   local($ch) = shift;
   if (&eqi('t',$ch)) {
      $testmode = 1;
   } elsif (&eqi('r',$ch)) {
      $recursive = 1;
   } elsif (&eqi('q',$ch)) {
      $quiet++;
   } elsif (&eqi('v',$ch)) {
      $verbose++;
   } else {
      &givehelp;
   }
}#see_an_option


@tmp = @ARGV;

# process command-line options
while (($#ARGV >= 0) && ('-' eq substr($ARGV[0],0,1))) {
   foreach $ch (split(//,substr($ARGV[0],1))) {
      &see_an_option( $ch );
   }
   shift @ARGV;
}

if ($verbose) {
   print "REM  #ARGV=$#ARGV,  @ARGV = '" . join("','",@tmp) . "'\n";
   print "REM  -v (verbose) = $verbose\n";
   print "REM  -q (quiet) = $quiet\n";
   print "REM  -r (recursive) = $recursive\n";
   print "REM  -t (testmode) = $testmode\n";
}


# if they didn't specify at least one directory, it is an error
if (-1 == $#ARGV) {
   &givehelp;
}


# create the special TLIB configuration file, and point %TLIBCFG% at it:
open( CFG, '>C:\RENLIBS.CFG' ) || die "ERROR: could not create C:\\RENLIBS.CFG, $!\n";
print CFG "commands L/L,C,CP,Q,?\n" .
          "workdir\n" .
          "treedirs_n\n" .
          "projlev\n" .
          "track_n\n" .
          "jfile\n" .
          "queries n\n";
close CFG;
$ENV{'TLIBCFG'} = '!C:\RENLIBS.CFG';


# Given a filename as input, return a 2-element array containing the base name
# and the extension.  The "." is removed.
sub split_fname {
  local( $fname ) = shift;
  local( @result ) = ('','');
  local( $dotpos ) = -1;
  $dotpos = rindex( $fname, '.' );
  if (-1 == $dotpos) {
     @result = ($fname,'')
  } else {
     @result = (substr($fname,0,$dotpos), substr($fname,$dotpos+1));
  }
  # print "dbg: split_fname('$fname')=('$result[0]','$result[1]')\n";
  return @result;
}#split_fname


# Given a path\name.ext as input, return a 2-element array containing path\ as
# the first element and name.ext as the 2nd element.  The '\' is not removed.
sub split_path{
  local( $fpath ) = shift;
  local( @result ) = ('','');
  local( $slashpos, $ndx2 );
  $slashpos = rindex($fpath,':');
  $ndx2 = rindex($fpath,"/");
  if ($ndx2 > $slashpos) {
     $slashpos = $ndx2;
  }
  $ndx2 = rindex($fpath,"\\");
  if ($ndx2 > $slashpos) {
     $slashpos = $ndx2;
  }
  if (-1 == $slashpos) {
     @result = ('',$fpath);
  } else {
     @result = (substr($fpath,0,$slashpos+1), substr($fpath,$slashpos+1));
  }
  # print "dbg: split_path('$fpath')=('$result[0]','$result[1]')\n";
  return @result;
}#split_path


# input is a source file extension and either $old_libext or $new_libext
# output is the resulting library file extension
sub apply_libext {
  local( $srcext, $libextpat ) = @_;
  local( $len ) = length( $libextpat );
  local( $result, $c1, $i );
  # split libext into an array of characters
  local( @tmp ) = split( //, $libextpat );
  # prune any extra '?'s from the end
  for ($i=$len-1; $i >= length($srcext); $i--) {
     if ('?' eq $tmp[$i]) {
        pop( @tmp );  # discard the trailing '?'
        $len--;
     } else {
        last;  # not a '?' so we're done
     }
  }#for
  # substitute from $srcext for the '?'s in $libextpat
  for ($i=0; $i < $len; $i++) {
     if ('?' eq $tmp[$i]) {
        if ($i >= length($srcext)) {
           $c1 = '_';
        } else {
           $c1 = substr( $srcext, $i, 1 );
           if (' ' eq $c1) {
              $c1 = '_';
           }
        }
        $tmp[$i] = $c1;
     }
  }#for
  # assemble the resulting array of characters into a string
  $result = join( '', @tmp );
  # print "dbg: apply_libext('$srcext','$libextpat')='$result'\n";
  return $result;
}#apply_libext


# input is a source file name with extension, and either $old_libext or $new_libext
# output is the resulting library file name
sub calc_libname {
  local( $srcname, $libextpat ) = @_;
  local( $base, $ext ) = &split_fname( $srcname );
  local( $result ) = $base . '.' . &apply_libext( $ext, $libextpat );
  return $result;
}#calc_libname


# case-insensitive 'eq'
sub eqi {
  local( $s1, $s2 ) = @_;
  if ($s1 eq $s2) {
     return 1;
  }
  $s1 =~ tr/a-z/A-Z/;
  $s2 =~ tr/a-z/A-Z/;
  return ($s1 eq $s2);
}#eqi


# Given the library file path\name, discover the source file name by listing
# the versions and parsing TLIB's output.
sub sourcename {
  local( $libpath ) = shift;
  local( $cmd, $tmp, $tmpL );
  local( $result ) = '';
  local( $warnmsg ) = '';
  # $libname is is just the name.ext, $libpath is the path\name.ext
  local( $junk, $libname );
  ($junk, $libname) = &split_path( $libpath );
  $cmd = 'tlib32c cp ' . &aq($libpath) . ' ll x';
  if ($verbose) {
     print "$cmd\n";
  }
  if (!open(TLIB,"$cmd|")) {
     print "REM - ERROR: could not run '$cmd', $!\n";
  } else {
     $ln = <TLIB>;
     if ($verbose >= 1) {print $ln;}
     if ((defined $ln) && ($ln =~ /^TLIB /)) {
        $ln = <TLIB>;
     if ($verbose >= 2) {print $ln;}
        if ((defined $ln) && ($ln =~ /Temporary license expires/)) {
           $ln = <TLIB>;
           if ($verbose >= 2) {print $ln;}
        }
        while ((defined $ln) && ($ln =~ /^\w*$/)) {  # skip blank lines
           $ln = <TLIB>;
        }
        if ((defined $ln) && ($ln =~ /^TLIB LL x \(library=/)) {
           while ($ln = <TLIB>) {
              if ($verbose >= 2) {print $ln;}
              # If we didn't have to worry about blanks in file names, we could
              # use this simpler regular expression:
              #              <---version----> <-name>
              #  ($ln =~ /^ ?([0-9\.\:\(\)]+) ([^ ]+) /)
              # This one determines the end of the source file name by looking
              # for the datestamp which always follows it:
              #                                          <---------------------------------date--------------------------------->
              #              <---version----> <--name-->  <-day->  <------------month----------->  <-----------year------------>
              if ($ln =~ /^ ?([0-9\.\:\(\)]+) ([^\,\/]+) (\d{1,2}\-[ADFJMNOS][aceopu][bcglnprtvy]\-(([890]\d)|(19\d\d)|(20\d\d)))[\,\/]\d/) {
                 # Since TLIB version 2.96 (circa Sept. 1986), the date is
                 # followed by a slash or comma and then the time.  To use
                 # this script with libraries built by TLIB 2.95 or earlier,
                 # substitute the simpler regular expression above, because
                 # you don't have to worry about blanks in file names.
                 if ($verbose == 1) {print $ln;}

                 # $1 is the version number, $2 is the source file name
                 $tmp = $2;

                 if ($verbose >= 2) {
                    print "dbg: version='$1' name='$2' date='$3'\n";
                 }

                 # If source file name we found this time is different from the
                 # one we found last time, see if source name is consisetent
                 # with the name of the library file
                 if ($tmp ne $result) {
                    $tmpL = &calc_libname( $tmp, $old_libext );
                    if (&eqi($tmpL,$libname)) {
                       $result = $tmp;
                    } else {
                       $warnmsg = "REM - Name found in library is \"$tmp\" which is inconsistent with library name \"$libname\"\n";
                    }
                 }
              }
           }
        }
     }
     close TLIB;
  }
  if (('' eq $result) && ('' ne $warnmsg)) {
     print $warnmsg;
  }
  return $result;
}#sourcename


# if path contains blanks, then add quote marks around it
sub aq {
   local( $path ) = shift;
   if (($path =~ / /) && ($path !~ /\"/)) {
      $path = '"' . $path . '"';
   }
   return $path;
}#aq


# rename a library file, if it is a library file, and if it hasn't already been renamed
sub ren_one_library_file {
  local( $fpath ) = shift;
  local( $dir, $fname ) = &split_path( $fpath );
  local( $newname, $newpath, $src, $readonly, $rc1, $rc2, $rc3 );
  if ((-f $fpath) && (-s $fpath)) {
     # check whether the name looks like a library file name
     $newname = &calc_libname( $fname, $old_libext );
     # if $fname is a library file, then $newname and $fname will be the same
     if (!&eqi($newname,$fname)) {
        if (!$quiet) {
           $newname = &calc_libname( $fname, $new_libext );
           if (!&eqi($newname,$fname)) {
              print "REM '$fname' is not a TLIB library file.\n";
           } else {
              print "REM '$fname' was already renamed.\n";
           }
        }
     } else {
        $src = &sourcename( $fpath );
        if ('' eq $src) {
           print "REM - ERROR: could not deduce source file name for library '$fpath'\n";
        } else {
           $readonly = (! -w $fpath);
           $newname = &calc_libname( $src, $new_libext );
           $newpath = ($dir . $newname);
           if ($testmode) {
              if (!$quiet) {
                 print "REM " . &aq($src) . "\n";
              }
              if ($readonly) {
                 print "attrib -r " . &aq($fpath) . "\n";
              }
              print "ren " . &aq($fpath) . " " . &aq($newname) . "\n";  # DOS-stype, specify path only fir src operand
              if ($readonly) {
                 print "attrib +r " . &aq($newpath) . "\n";
              }
           } else {
              if (!$quiet) {
                 print "ren " . &aq($fname) . " " . &aq($newname) . "\n";
              }
              if ($readonly) {
                 $rc1 = chmod( 0666, $fpath);  # writable
                 if ($verbose) {
                    print "dbg: chmod(0666,'$fname')=$rc1\n";
                 }
                 if (1 != $rc1) {
                    print "ERROR: 'attrib -r " . &aq($fname) . "' failed, $!\n";
                 }
              }
              $rc2 = rename( $fpath, $newpath );  # Unix-style, specify path for both operands
              if ($verbose) {
                 print "dbg: rename('$fpath','$newpath')=$rc2\n";
              }
              if (1 != $rc2) {
                 print "ERROR: 'ren " . &aq($fpath) . " " . &aq($newpath) . "' failed, $!\n";
              }
              if ($readonly) {
                 $rc3 = chmod( 0444, $newpath);  # read-only
                 if ($verbose) {
                    print "dbg: chmod(0444,'$newpath')=$rc3\n";
                 }
                 if (1 != $rc3) {
                    print "ERROR: 'attrib -r " . &aq($newpath) . "' failed, $!\n";
                 }
              }
           }
        }
     }
  }
}#ren_one_library_file


# Remove trailing backslash from a directory name or path, unless it is the
# drive root ("\" or "c:\")
sub remove_trailing_slash {
  local( $dir ) = shift;
  if ($verbose) {
     print "dbg: dir='$dir'\n";
  }
  if (length($dir) > 1) {
     # delete trailing quote mark, if any
     if ('"' eq substr($dir,length($dir)-1)) {
        $dir = substr( $dir, 0, length($dir)-1 );
        if ($verbose) {
           print "  dbg: changed to '$dir' (removed trailing quote)\n";
        }
     }
  }
  if (length($dir) > 1) {
     # delete leading quote mark, if any
     if ('"' eq substr($dir,1,1)) {
        $dir = substr( $dir, 1 );
        if ($verbose) {
           print "  dbg: changed to '$dir' (removed leading quote)\n";
        }
     }
  }
  if (length($dir) > 1) {
     # delete trailing slash or backslash (except for drive root)
     if (   ("\\" eq substr($dir,length($dir)-1))
         || ("/" eq substr($dir,length($dir)-1)) ) {
        if ((3 != length($dir)) || (':' ne substr($dir,1,1))) {
           $dir = substr( $dir, 0, length($dir)-1 );
           if ($verbose) {
              print "  dbg: changed to '$dir' (removed trailing slash)\n";
           }
        }
     }
  }
  return $dir;
}#remove_trailing_slash


# Add trailing backslash to a directory name or path, unless it is already there
sub add_trailing_slash {
  local( $dir ) = shift;
  if ('' ne $dir) {
     if (   ("\\" ne substr($dir,length($dir)-1))
         && ("/" ne substr($dir,length($dir)-1)) ) {
        $dir .= "\\";
     }
  }
  return $dir;
}#add_trailing_slash



@directories = @ARGV;

while (defined ($curdir=shift(@directories))) {
   $curdir = &remove_trailing_slash( $curdir );
   opendir( DIR, $curdir ) || die "ERROR: Could not read directory '$curdir', $!\n";
   if (! $quiet) {
      print "\nREM -- directory $curdir --\n";
   }
   $curdir = &add_trailing_slash( $curdir );
   @files_in_this_dir = ();
   while ($fname = readdir(DIR)) {
      $path = $curdir . $fname;
      if (-d $path) {
         # found a subdirectory name
         if ($recursive && ('.' ne $fname) && ('..' ne $fname)) {
            push( @directories, $path );
         }
      } elsif ((-f $path) && (-s $path)) {
         push( @files_in_this_dir, $path );
      }
   }
   closedir(DIR);
   foreach $path (@files_in_this_dir) {
      &ren_one_library_file( $path );
   }
}#while


if (! $quiet) {
   print "done.\n";
}
# clean-up
unlink 'C:\RENLIBS.CFG';
exit 0;

__END__
